package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.NftcanStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NftcanStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface NftcanStructMapper extends IBaseStructMapper<NftcanStoreBo, NftcanStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NftcanStructMapper INSTANCE = Mappers.getMapper(NftcanStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NftcanStoreBo voTransferToBo(NftcanStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mapping(target = "nidgrp", ignore = true)
    @Mapping(target = "ncdgrp", ignore = true)
    void voUpdateToBo(NftcanStoreVo vo, @MappingTarget NftcanStoreBo bo);

    @AfterMapping
    default void afterUpdate(NftcanStoreVo vo, @MappingTarget NftcanStoreBo bo) {
        bo.getNcdgrp().getRec().setClsdat(vo.getNcdgrp().getRec().getClsdat());
        bo.setRebkpflg(vo.getRebkpflg());
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NftcanStoreVo boTransferToVo(NftcanStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NftcanStoreBo bo, @MappingTarget NftcanStoreVo vo);
}
